@@ -74,6 +74,11 @@ CURL *          builder_context_get_curl_session (BuilderContext *self);
 const char *    builder_context_get_arch (BuilderContext *self);
 void            builder_context_set_arch (BuilderContext *self,
                                           const char     *arch);
+const char *    builder_context_get_default_branch (BuilderContext *self);
+void            builder_context_set_default_branch (BuilderContext *self,
+                                                    const char     *default_branch);
+void            builder_context_set_source_date_epoch (BuilderContext *self,
+                                                       gint64 source_date_epoch);
 const char *    builder_context_get_stop_at (BuilderContext *self);
 void            builder_context_set_stop_at (BuilderContext *self,
                                              const char     *module);
@@ -140,8 +145,10 @@ void            builder_context_set_run_tests (BuilderContext *self,
 void            builder_context_set_no_shallow_clone (BuilderContext *self,
                                                       gboolean        no_shallow_clone);
 gboolean        builder_context_get_no_shallow_clone (BuilderContext *self);
-char **         builder_context_extend_env (BuilderContext *self,
-                                            char          **envp);
+char **         builder_context_extend_env_pre (BuilderContext *self,
+                                                 char          **envp);
+char **         builder_context_extend_env_post (BuilderContext *self,
+                                                 char          **envp);
 
 gboolean        builder_context_load_sdk_config (BuilderContext       *self,
                                                  const char           *sdk_path,
