@@ -565,6 +565,8 @@ flatpak_spawnv (GFile                *dir,
 
   launcher = g_subprocess_launcher_new (0);
 
+  g_subprocess_launcher_setenv (launcher, "LANGUAGE", "C", TRUE);
+
   if (output)
     flags |= G_SUBPROCESS_FLAGS_STDOUT_PIPE;
 
@@ -693,6 +695,7 @@ gboolean
 flatpak_cp_a (GFile         *src,
               GFile         *dest,
               FlatpakCpFlags flags,
+              GPtrArray     *skip_files,
               GCancellable  *cancellable,
               GError       **error)
 {
@@ -763,14 +766,29 @@ flatpak_cp_a (GFile         *src,
     {
       const char *name = g_file_info_get_name (child_info);
       g_autoptr(GFile) src_child = g_file_get_child (src, name);
+      gboolean skip = FALSE;
+      int i;
+
+      for (i = 0; skip_files != NULL && i < skip_files->len; i++)
+        {
+          if (g_file_equal (src_child, g_ptr_array_index (skip_files, i)))
+            {
+              skip = TRUE;
+              break;
+            }
+        }
 
       if (dest_child)
         g_object_unref (dest_child);
       dest_child = g_file_get_child (dest, name);
 
-      if (g_file_info_get_file_type (child_info) == G_FILE_TYPE_DIRECTORY)
+      if (skip)
         {
-          if (!flatpak_cp_a (src_child, dest_child, flags,
+          /* skip src */
+        }
+      else if (g_file_info_get_file_type (child_info) == G_FILE_TYPE_DIRECTORY)
+        {
+          if (!flatpak_cp_a (src_child, dest_child, flags, skip_files,
                              cancellable, error))
             goto out;
         }
@@ -1055,137 +1073,6 @@ flatpak_allocate_tmpdir (int           tmpdir_dfd,
 }
 
 
-typedef struct {
-  GMainLoop *loop;
-  GError *error;
-  GOutputStream *out;
-  guint64 downloaded_bytes;
-  GString *content;
-  char buffer[16*1024];
-  FlatpakLoadUriProgress progress;
-  GCancellable *cancellable;
-  gpointer user_data;
-  guint64 last_progress_time;
-  char *etag;
-} LoadUriData;
-
-static void
-stream_closed (GObject *source, GAsyncResult *res, gpointer user_data)
-{
-  LoadUriData *data = user_data;
-  GInputStream *stream = G_INPUT_STREAM (source);
-  g_autoptr(GError) error = NULL;
-
-  if (!g_input_stream_close_finish (stream, res, &error))
-    g_warning ("Error closing http stream: %s", error->message);
-
-  g_main_loop_quit (data->loop);
-}
-
-static void
-load_uri_read_cb (GObject *source, GAsyncResult *res, gpointer user_data)
-{
-  LoadUriData *data = user_data;
-  GInputStream *stream = G_INPUT_STREAM (source);
-  gsize nread;
-
-  nread = g_input_stream_read_finish (stream, res, &data->error);
-  if (nread == -1 || nread == 0)
-    {
-      if (data->progress)
-        data->progress (data->downloaded_bytes, data->user_data);
-      g_input_stream_close_async (stream,
-                                  G_PRIORITY_DEFAULT, NULL,
-                                  stream_closed, data);
-      return;
-    }
-
-  if (data->out != NULL)
-    {
-      gsize n_written;
-
-      if (!g_output_stream_write_all (data->out, data->buffer, nread, &n_written,
-                                      NULL, &data->error))
-        {
-          data->downloaded_bytes += n_written;
-          g_input_stream_close_async (stream,
-                                      G_PRIORITY_DEFAULT, NULL,
-                                      stream_closed, data);
-          return;
-        }
-
-      data->downloaded_bytes += n_written;
-    }
-  else
-    {
-      data->downloaded_bytes += nread;
-      g_string_append_len (data->content, data->buffer, nread);
-    }
-
-  if (g_get_monotonic_time () - data->last_progress_time > 1 * G_USEC_PER_SEC)
-    {
-      if (data->progress)
-        data->progress (data->downloaded_bytes, data->user_data);
-      data->last_progress_time = g_get_monotonic_time ();
-    }
-
-  g_input_stream_read_async (stream, data->buffer, sizeof (data->buffer),
-                             G_PRIORITY_DEFAULT, data->cancellable,
-                             load_uri_read_cb, data);
-}
-
-static void
-load_uri_callback (GObject *source_object,
-                   GAsyncResult *res,
-                   gpointer user_data)
-{
-  SoupRequestHTTP *request = SOUP_REQUEST_HTTP(source_object);
-  g_autoptr(GInputStream) in = NULL;
-  LoadUriData *data = user_data;
-
-  in = soup_request_send_finish (SOUP_REQUEST (request), res, &data->error);
-  if (in == NULL)
-    {
-      g_main_loop_quit (data->loop);
-      return;
-    }
-
-  g_autoptr(SoupMessage) msg = soup_request_http_get_message ((SoupRequestHTTP*) request);
-  if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
-    {
-      int code;
-      GQuark domain = G_IO_ERROR;
-
-      switch (msg->status_code)
-        {
-        case 304:
-          //TODO          domain = FLATPAK_OCI_ERROR;
-          //TODO          code = FLATPAK_OCI_ERROR_NOT_CHANGED;
-          code = G_IO_ERROR_FAILED;
-          break;
-        case 404:
-        case 410:
-          code = G_IO_ERROR_NOT_FOUND;
-          break;
-        default:
-          code = G_IO_ERROR_FAILED;
-        }
-
-      data->error = g_error_new (domain, code,
-                                 "Server returned status %u: %s",
-                                 msg->status_code,
-                                 soup_status_get_phrase (msg->status_code));
-      g_main_loop_quit (data->loop);
-      return;
-    }
-
-  data->etag = g_strdup (soup_message_headers_get_one (msg->response_headers, "ETag"));
-
-  g_input_stream_read_async (in, data->buffer, sizeof (data->buffer),
-                             G_PRIORITY_DEFAULT, data->cancellable,
-                             load_uri_read_cb, data);
-}
-
 SoupSession *
 flatpak_create_soup_session (const char *user_agent)
 {
@@ -1212,52 +1099,25 @@ flatpak_create_soup_session (const char *user_agent)
   return soup_session;
 }
 
-gboolean
-flatpak_download_http_uri (SoupSession *soup_session,
-                           const char   *uri,
-                           GOutputStream *out,
-                           FlatpakLoadUriProgress progress,
-                           gpointer      user_data,
-                           GCancellable *cancellable,
-                           GError      **error)
+CURL *
+flatpak_create_curl_session (const char *user_agent)
 {
-  g_autoptr(SoupRequestHTTP) request = NULL;
-  g_autoptr(GMainLoop) loop = NULL;
-  g_autoptr(GMainContext) context = NULL;
-  LoadUriData data = { NULL };
+  CURL *curl_session;
 
-  g_debug ("Loading %s using libsoup", uri);
+  curl_global_init (CURL_GLOBAL_DEFAULT);
 
-  context = g_main_context_ref_thread_default ();
+  curl_session = curl_easy_init ();
+  if (curl_session == NULL)
+    return NULL;
 
-  loop = g_main_loop_new (context, TRUE);
-  data.loop = loop;
-  data.out = out;
-  data.progress = progress;
-  data.cancellable = cancellable;
-  data.user_data = user_data;
-  data.last_progress_time = g_get_monotonic_time ();
+  curl_easy_setopt (curl_session, CURLOPT_CONNECTTIMEOUT, 60);
+  curl_easy_setopt (curl_session, CURLOPT_FAILONERROR, 1);
+  curl_easy_setopt (curl_session, CURLOPT_FOLLOWLOCATION, 1);
+  curl_easy_setopt (curl_session, CURLOPT_MAXREDIRS, 50);
+  curl_easy_setopt (curl_session, CURLOPT_NOPROGRESS, 0);
+  curl_easy_setopt (curl_session, CURLOPT_USERAGENT, user_agent);
 
-  request = soup_session_request_http (soup_session, "GET",
-                                       uri, error);
-  if (request == NULL)
-    return FALSE;
-
-  soup_request_send_async (SOUP_REQUEST(request),
-                           cancellable,
-                           load_uri_callback, &data);
-
-  g_main_loop_run (loop);
-
-  if (data.error)
-    {
-      g_propagate_error (error, data.error);
-      return FALSE;
-    }
-
-  g_debug ("Received %" G_GUINT64_FORMAT " bytes", data.downloaded_bytes);
-
-  return TRUE;
+  return curl_session;
 }
 
 typedef enum {
