@@ -38,6 +38,7 @@ struct BuilderSourceFile
 
   char         *path;
   char         *url;
+  char        **mirror_urls;
   char         *md5;
   char         *sha1;
   char         *sha256;
@@ -61,6 +62,7 @@ enum {
   PROP_SHA256,
   PROP_SHA512,
   PROP_DEST_FILENAME,
+  PROP_MIRROR_URLS,
   LAST_PROP
 };
 
@@ -76,6 +78,7 @@ builder_source_file_finalize (GObject *object)
   g_free (self->sha256);
   g_free (self->sha512);
   g_free (self->dest_filename);
+  g_strfreev (self->mirror_urls);
 
   G_OBJECT_CLASS (builder_source_file_parent_class)->finalize (object);
 }
@@ -118,6 +121,10 @@ builder_source_file_get_property (GObject    *object,
       g_value_set_string (value, self->dest_filename);
       break;
 
+    case PROP_MIRROR_URLS:
+      g_value_set_boxed (value, self->mirror_urls);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -130,6 +137,7 @@ builder_source_file_set_property (GObject      *object,
                                   GParamSpec   *pspec)
 {
   BuilderSourceFile *self = BUILDER_SOURCE_FILE (object);
+  gchar **tmp;
 
   switch (prop_id)
     {
@@ -168,6 +176,12 @@ builder_source_file_set_property (GObject      *object,
       self->dest_filename = g_value_dup_string (value);
       break;
 
+    case PROP_MIRROR_URLS:
+      tmp = self->mirror_urls;
+      self->mirror_urls = g_strdupv (g_value_get_boxed (value));
+      g_strfreev (tmp);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -276,9 +290,9 @@ get_source_file (BuilderSourceFile *self,
 }
 
 static GBytes *
-download_uri (const char     *url,
-              BuilderContext *context,
-              GError        **error)
+download_data_uri (const char     *url,
+                   BuilderContext *context,
+                   GError        **error)
 {
   SoupSession *session;
 
@@ -347,30 +361,22 @@ builder_source_file_download (BuilderSource  *source,
 
   if (g_file_query_exists (file, NULL))
     {
-      if (is_local && checksums[0] != NULL)
-        {
-          g_autofree char *data = NULL;
-          gsize len;
-
-          if (!g_file_load_contents (file, NULL, &data, &len, NULL, error))
-            return FALSE;
-
-          if (!builder_verify_checksums (base_name,
-                                         data, len,
-                                         checksums, checksums_type,
-                                         error))
-            return FALSE;
-        }
-      return TRUE;
+      return !is_local || checksums[0] == NULL ||
+             builder_verify_checksums (base_name, file,
+                                       checksums, checksums_type,
+                                       error);
     }
 
+  if (is_inline)
+    return TRUE;
+
   if (is_local)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Can't find file at %s", self->path);
       return FALSE;
     }
 
-  if (checksums[0] == NULL && !is_inline)
+  if (checksums[0] == NULL)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No checksum specified for file source %s", base_name);
       return FALSE;
@@ -378,6 +384,7 @@ builder_source_file_download (BuilderSource  *source,
 
   if (!builder_context_download_uri (context,
                                      self->url,
+                                     (const char **)self->mirror_urls,
                                      file,
                                      checksums,
                                      checksums_type,
@@ -432,9 +439,9 @@ builder_source_file_extract (BuilderSource  *source,
     {
       g_autoptr(GBytes) content = NULL;
 
-      content = download_uri (self->url,
-                              context,
-                              error);
+      content = download_data_uri (self->url,
+                                   context,
+                                   error);
       if (content == NULL)
         return FALSE;
 
@@ -562,6 +569,7 @@ builder_source_file_checksum (BuilderSource  *source,
   builder_cache_checksum_compat_str (cache, self->sha1);
   builder_cache_checksum_compat_str (cache, self->sha512);
   builder_cache_checksum_str (cache, self->dest_filename);
+  builder_cache_checksum_compat_strv (cache, self->mirror_urls);
 }
 
 static void
@@ -630,6 +638,13 @@ builder_source_file_class_init (BuilderSourceFileClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_MIRROR_URLS,
+                                   g_param_spec_boxed ("mirror-urls",
+                                                       "",
+                                                       "",
+                                                       G_TYPE_STRV,
+                                                       G_PARAM_READWRITE));
 }
 
 static void
